package opus;

final class JAnalysisGRULayer extends Jmlp {
	private final byte[] bias;
	private final byte[] input_weights;
	private final byte[] recurrent_weights;
	private final int nb_inputs;
	private final int nb_neurons;

	JAnalysisGRULayer(final byte[] b, final byte[] iw, final byte[] rw, final int i, final int n) {
		bias = b;
		input_weights = iw;
		recurrent_weights = rw;
		nb_inputs = i;
		nb_neurons = n;
	}

	final void analysis_compute_gru(final float[] state, final float[] input)
	{
		final float tmp[] = new float[ MAX_NEURONS ];
		final float z[] = new float[ MAX_NEURONS ];
		final float r[] = new float[ MAX_NEURONS ];
		final float h[] = new float[ MAX_NEURONS ];
		int M = this.nb_inputs;
		int N = this.nb_neurons;
		int stride = 3 * N;
		final byte[] b = this.bias;// java
		/* Compute update gate. */
		for( int i = 0; i < N; i++ ) {
			z[ i ] = b[ i ];
		}
		gemm_accum( z, this.input_weights, 0, N, M, stride, input );
		gemm_accum( z, this.recurrent_weights, 0, N, N, stride, state );
		for( int i = 0; i < N; i++ ) {
			z[ i ] = .5f + .5f * tansig_approx( .5f * WEIGHTS_SCALE * z[ i ] );// sigmoid_approx( WEIGHTS_SCALE * z[ i ] );
		}

		/* Compute reset gate. */
		for( int i = 0, j = N; i < N; i++, j++ ) {
			r[ i ] = b[ j ];
		}
		gemm_accum( r, this.input_weights, N, N, M, stride, input );
		gemm_accum( r, this.recurrent_weights, N, N, N, stride, state );
		for( int i = 0; i < N; i++ ) {
			r[ i ] = .5f + .5f * tansig_approx( .5f * WEIGHTS_SCALE * r[ i ] );// sigmoid_approx( WEIGHTS_SCALE * r[ i ] );
		}
		/* Compute output. */
		for( int i = 0, j = (N << 1); i < N; i++, j++ ) {
			h[ i ] = b[ j ];
		}
		for( int i = 0; i < N; i++ ) {
			tmp[ i ] = state[ i ] * r[ i ];
		}
		gemm_accum( h, this.input_weights, N << 1, N, M, stride, input );
		gemm_accum( h, this.recurrent_weights, N << 1, N, N, stride, tmp );
		for( int i = 0; i < N; i++ ) {
			h[ i ] = z[ i ] * state[ i ] + (1f - z[ i ]) * tansig_approx( WEIGHTS_SCALE * h[ i ] );
		}
		for( int i = 0; i < N; i++ ) {
			state[ i ] = h[ i ];
		}
	}
}
