package opus;

/* Copyright (c) 2012 Xiph.Org Foundation
   Written by Jean-Marc Valin */

// opus_private.h

public final class Jopus_private {

	public static final int MODE_SILK_ONLY = 1000;
	static final int MODE_HYBRID    = 1001;
	public static final int MODE_CELT_ONLY = 1002;

	/** Configures the encoder's expected percentage of voice
	  * opposed to music or other signals.
	  *
	  * @note This interface is currently more aspiration than actuality. It's
	  * ultimately expected to bias an automatic signal classifier, but it currently
	  * just shifts the static bitrate to mode mapping around a little bit.
	  *
	  * @param[in] x <tt>int</tt>:   Voice percentage in the range 0-100, inclusive.
	  * @hideinitializer */
	static final int OPUS_SET_VOICE_RATIO_REQUEST = 11018;

	/** Gets the encoder's configured voice ratio value, @see OPUS_SET_VOICE_RATIO
	  *
	  * @param[out] x <tt>int*</tt>:  Voice percentage in the range 0-100, inclusive.
	  * @hideinitializer */
	static final int OPUS_GET_VOICE_RATIO_REQUEST = 11019;

	/** Configures the encoder's expected percentage of voice
	  * opposed to music or other signals.
	  *
	  * @note This interface is currently more aspiration than actuality. It's
	  * ultimately expected to bias an automatic signal classifier, but it currently
	  * just shifts the static bitrate to mode mapping around a little bit.
	  *
	  * @param[in] x <tt>int</tt>:   Voice percentage in the range 0-100, inclusive.
	  * @hideinitializer */
	// #define OPUS_SET_VOICE_RATIO(x) OPUS_SET_VOICE_RATIO_REQUEST, __opus_check_int(x)
	static final int OPUS_SET_VOICE_RATIO = OPUS_SET_VOICE_RATIO_REQUEST;

	/** Gets the encoder's configured voice ratio value, @see OPUS_SET_VOICE_RATIO
	  *
	  * @param[out] x <tt>int*</tt>:  Voice percentage in the range 0-100, inclusive.
	  * @hideinitializer */
	// #define OPUS_GET_VOICE_RATIO(x) OPUS_GET_VOICE_RATIO_REQUEST, __opus_check_int_ptr(x)
	static final int OPUS_GET_VOICE_RATIO = OPUS_GET_VOICE_RATIO_REQUEST;

	static final int  OPUS_SET_FORCE_MODE_REQUEST = 11002;
	// #define OPUS_SET_FORCE_MODE(x) OPUS_SET_FORCE_MODE_REQUEST, __opus_check_int(x)
	public static final int  OPUS_SET_FORCE_MODE = OPUS_SET_FORCE_MODE_REQUEST;

	// extensions.c
	/**
	 * Java data holder for using with {@link #skip_extension}
	 */
	private static final class Jextension_aux {
		// private final byte[] mData;
		private int mOffset;
		private int mHeaderSize;

		// private Jextension_aux(final byte[] data) {
		//	mData = data;
		// }
	}

	/** Given an extension payload, advance data to the next extension and return the
	   length of the remaining extensions.

	   More than that is ridiculous for now (3 * max frames per packet)
	 */
	// static int skip_extension(final byte[] data, final int doff, int len, final int[] header_size) {
	static int skip_extension(final byte[] data, final Jextension_aux aux, int len)
	{// java: Jextension_aux is used instead of unsigned char **data and opus_int32 *header_size
		if( len == 0 ) {
			return 0;
		}
		int L = data[ aux.mOffset ];
		final int id = (L & 0xFF) >> 1;
		L &= 1;
		if( id == 0 && L == 1 ) {
			aux.mHeaderSize = 1;
			if( len < 1 ) {
				return -1;
			}
			aux.mOffset++;
			len--;
			return len;
		} else if( id > 0 && id < 32 )
		{
			if( len < 1 + L ) {
				return -1;
			}
			aux.mOffset += 1 + L;
			len -= 1 + L;
			aux.mHeaderSize = 1;
			return len;
		} else {
			if( L == 0 ) {
				aux.mOffset += len;
				aux.mHeaderSize = 1;
				return 0;
			} else {
				int di;// java
				int bytes = 0;
				aux.mHeaderSize = 1;
				do {
					aux.mOffset++;
					len--;
					if( len == 0 ) {
						return -1;
					}
					di = (data[ aux.mOffset ] & 0xFF);
					bytes += di;
					aux.mHeaderSize++;
				} while( di == 255 );
				aux.mOffset++;
				len--;
				if( bytes <= len ) {
					len -= bytes;
					aux.mOffset += bytes;
					return len;
				}// else {
					return -1;
				// }
				// return len;
			}
		}
	}

	/** Count the number of extensions, excluding real padding and separators. */
	public static int opus_packet_extensions_count(final byte[] data, final int doff, final int len) {
		int count = 0;
		int curr_data = doff;// java offset to data;

		// celt_assert(len >= 0);
		// celt_assert(data != NULL || len == 0);

		final Jextension_aux aux = new Jextension_aux();// java
		int curr_len = len;
		while( curr_len > 0 ) {
			// final int header_size; java: is not used here
			final int id = ((int)data[ curr_data ] & 0xFF) >> 1;
			aux.mOffset = curr_data;
			curr_len = skip_extension( data, aux, curr_len );
			curr_data = aux.mOffset;
			if( curr_len < 0 ) {
				return Jopus_defines.OPUS_INVALID_PACKET;
			}
			if( id > 1 ) {
				count++;
			}
		}
		return count;
	}

	/** Extract extensions from Opus padding (excluding real padding and separators) */
	public static int opus_packet_extensions_parse(final byte[] data, final int doff, final int len,
			final Jopus_extension_data[] extensions, final int eoff,// java
			final int[] nb_extensions)
	{
		int curr_frame = 0;
		int count = 0;

		// celt_assert(len >= 0);
		// celt_assert(data != NULL || len == 0);
		// celt_assert(nb_extensions != NULL);
		// celt_assert(extensions != NULL || *nb_extensions == 0);

		final Jextension_aux aux = new Jextension_aux();// java
		int curr_data = doff;// java: an offset to data;
		int curr_len = len;
		while( curr_len > 0 ) {
			final Jopus_extension_data curr_ext = new Jopus_extension_data();
			curr_ext.data_base = data;// java
			final int id = (data[curr_data] & 0xFF) >> 1;
			if( id > 1 ) {
				curr_ext.id = id;
				curr_ext.frame = curr_frame;
				curr_ext.data = curr_data;
			} else if( id == 1 )
			{
				final int L = data[curr_data] & 1;
				if( L == 0 ) {
					curr_frame++;
				} else {
					if( curr_len >= 2 ) {
						curr_frame += (data[ curr_data + 1 ] & 0xFF);
					}
					/* Else we're at the end and it doesn't matter. */
				}
				if( curr_frame >= 48 ) {
					nb_extensions[0] = count;
					return Jopus_defines.OPUS_INVALID_PACKET;
				}
			}

			// final int header_size;
			// curr_len = skip_extension( data, curr_data, curr_len, header_size );
			aux.mOffset = curr_data;
			curr_len = skip_extension( data, aux, curr_len );
			curr_data = aux.mOffset;
			final int header_size = aux.mHeaderSize;

			/* printf("curr_len = %d, header_size = %d\n", curr_len, header_size); */
			if( curr_len < 0 ) {
				nb_extensions[0] = count;
				return Jopus_defines.OPUS_INVALID_PACKET;
			}
			// celt_assert(curr_data - data == len - curr_len);
			if( id > 1 ) {
				if( count == nb_extensions[0] ) {
					return Jopus_defines.OPUS_BUFFER_TOO_SMALL;
				}
				curr_ext.len = curr_data - curr_ext.data - header_size;
				curr_ext.data += header_size;
				extensions[eoff + count++] = curr_ext;
			}
		}
		// celt_assert(curr_len == 0);
		nb_extensions[ 0 ] = count;
		return Jopus_defines.OPUS_OK;
	}

	public static int opus_packet_extensions_generate(final byte[] data, final int doff, final int len, final Jopus_extension_data[] extensions, final int nb_extensions, final int pad)
	{
		int max_frame = 0;
		int curr_frame = 0;
		int pos = 0;
		int written = 0;

		// celt_assert(len >= 0);

		for( int i = 0; i < nb_extensions; i++ ) {
			final Jopus_extension_data ei = extensions[i];
			if( max_frame < ei.frame ) {
				max_frame = ei.frame;
			}
			if( ei.id < 2 || ei.id > 127 ) {
				return Jopus_defines.OPUS_BAD_ARG;
			}
		}
		if( max_frame >= 48 ) {
			return Jopus_defines.OPUS_BAD_ARG;
		}
		for( int frame = 0; frame <= max_frame; frame++) {
			for( int i = 0; i < nb_extensions; i++ ) {
				final Jopus_extension_data ei = extensions[i];
				if( ei.frame == frame ) {
					/* Insert separator when needed. */
					if( frame != curr_frame ) {
						final int diff = frame - curr_frame;
						if( len - pos < 2 ) {
							return Jopus_defines.OPUS_BUFFER_TOO_SMALL;
						}
						if( diff == 1 ) {
							if( null != data ) {
								data[doff + pos] = 0x02;
							}
							pos++;
						} else {
							if( null != data ) {
								data[doff + pos] = 0x03;
							}
							pos++;
							if( null != data ) {
								data[doff + pos] = (byte) diff;
							}
							pos++;
						}
						curr_frame = frame;
					}
					if( ei.id < 32 ) {
						if( ei.len < 0 || ei.len > 1 ) {
							return Jopus_defines.OPUS_BAD_ARG;
						}
						if( len - pos < ei.len + 1 ) {
							return Jopus_defines.OPUS_BUFFER_TOO_SMALL;
						}
						if( null != data ) {
							data[doff + pos] = (byte)((ei.id << 1) + ei.len);
						}
						pos++;
						if( ei.len > 0 ) {
							if( null != data ) {
								data[doff + pos] = ei.data_base[ ei.data ];// java ei.data[0]; TODO verify using ei.data_base
							}
							pos++;
						}
					} else {
						if( ei.len < 0 ) {
							return Jopus_defines.OPUS_BAD_ARG;
						}
						final boolean last = (written == nb_extensions - 1);
						int length_bytes = 1 + ei.len / 255;
						if( last ) {
							length_bytes = 0;
						}
						if( len - pos < 1 + length_bytes + ei.len ) {
							return Jopus_defines.OPUS_BUFFER_TOO_SMALL;
						}
						if( null != data ) {
							data[doff + pos] = (byte) ((ei.id << 1) + (last ? 0 : 1));
						}
						pos++;
						if( ! last ) {
							for( int j = 0, je = ei.len / 255; j < je; j++ ) {
								if( null != data ) {
									data[doff + pos] = (byte)255;
								}
								pos++;
							}
							if( null != data ) {
								data[doff + pos] = (byte)(ei.len % 255);
							}
							pos++;
						}
						if( null != data ) {
							System.arraycopy( ei.data_base, ei.data, data, doff + pos, ei.len );
						}
						pos += ei.len;
					}
					written++;
				}
			}
		}
		/* If we need to pad, just prepend 0x01 bytes. Even better would be to fill the
		  end with zeros, but that requires checking that turning the last extesion into
		  an L=1 case still fits. */
		if( (0 != pad) && (pos < len) ) {
			final int padding = len - pos;
			if( null != data ) {
				final int dp = doff + padding;// java
				System.arraycopy( data, doff, data, dp, pos );
				for( int i = doff; i < dp; i++ ) {
					data[i] = 0x01;
				}
			}
			pos += padding;
		}
		return pos;
	}
}