package test;

import java.util.Arrays;

import opus.JOpusRepacketizer;
import opus.Jopus;
import opus.Jopus_defines;
import opus.Jopus_extension_data;
import opus.Jopus_packet_data_aux;
import opus.Jopus_private;

/* Copyright (c) 2023 Amazon
   Written by Michael Klingbeil */
/*
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

final class Jtest_opus_extensions {
	private static final String CLASS_NAME = "Jtest_opus_extensions";

	private static void test_extensions_generate_success() {
		final String method_name = "test_extensions_generate_success";
		final Jopus_extension_data[] ext = {
				new Jopus_extension_data( 2, 0, "a".getBytes(), 0, 1 ),
				new Jopus_extension_data( 32, 10, "DRED".getBytes(), 0, 4 ),
				new Jopus_extension_data( 33, 1, "NOT DRED".getBytes(), 0, 8 ),
				new Jopus_extension_data( 3, 4, null, 0, 0 )
			};
		final byte[] packet = new byte[32];
		int p = 0;// java: packet[ p ];
		final int result = Jopus_private.opus_packet_extensions_generate( packet, 0, 23+4, ext, 4, 1);
		Jtest_opus_common.expect_true( result == 23+4, "expected length 23+4", CLASS_NAME, method_name );

		/* expect padding */
		Jtest_opus_common.expect_true(packet[p+0] == 1 && packet[p+1] == 1 && packet[p+2] == 1 && packet[p+3] == 1, "expected padding", CLASS_NAME, method_name);
		p += 4;

		/* extension ID=2 */
		Jtest_opus_common.expect_true((packet[p+0] >> 1) == 2, "expected extension id 2", CLASS_NAME, method_name);
		/* For extension IDs 1 through 31, L=0 means that no data follows the
		  extension, whereas L=1 means that exactly one byte of extension data follows. */
		Jtest_opus_common.expect_true((packet[p+0] & 0x01) == 1, "expected L-bit set", CLASS_NAME, method_name);
		/* content */
		Jtest_opus_common.expect_true(packet[p+1] == 'a', "expected extension content", CLASS_NAME, method_name);
		p += 2;

		/* next byte should increment the frame count, ID=1, L=0 */
		Jtest_opus_common.expect_true(packet[p+0] == 0x02, "bad frame separator", CLASS_NAME, method_name);
		p += 1;
		/* extension ID=33 */
		Jtest_opus_common.expect_true((packet[p+0] >> 1) == 33, "expected extension id 33", CLASS_NAME, method_name);
		/* For IDs 32 to 127, L=0 signals that the extension data takes up the
		  rest of the padding, and L=1 signals that a length indicator follows. */
		Jtest_opus_common.expect_true((packet[p+0] & 0x01) == 1, "expected L-bit set", CLASS_NAME, method_name);
		/* content */
		Jtest_opus_common.expect_true(packet[p+1] == ext[2].len, "expected length", CLASS_NAME, method_name);
		p += 2;
		Jtest_opus_common.expect_true(0 == Jtest_opus_common.memcmp(packet,p, ext[2].data_base,ext[2].data, ext[2].len), "expected extension content", CLASS_NAME, method_name);
		p += ext[2].len;

		/* advance to frame 4, increment by 3 */
		/* next byte should increment the frame count, ID=1, L=1 */
		Jtest_opus_common.expect_true(packet[p+0] == 0x03, "bad frame separator", CLASS_NAME, method_name);
		Jtest_opus_common.expect_true(packet[p+1] == 0x03, "bad frame increment", CLASS_NAME, method_name);
		p += 2;
		/* extension ID=3 */
		Jtest_opus_common.expect_true((packet[p+0] >> 1) == 3, "expected extension id 3", CLASS_NAME, method_name);
		/* For extension IDs 1 through 31, L=0 means that no data follows the
		  extension, whereas L=1 means that exactly one byte of extension data follows. */
		Jtest_opus_common.expect_true((packet[p+0] & 0x01) == 0, "expected L-bit unset", CLASS_NAME, method_name);
		p += 1;

		/* advance to frame 10, increment by 6 */
		/* next byte should increment the frame count, ID=1, L=1 */
		Jtest_opus_common.expect_true(packet[p+0] == 0x03, "bad frame separator", CLASS_NAME, method_name);
		Jtest_opus_common.expect_true(packet[p+1] == 0x06, "bad frame increment", CLASS_NAME, method_name);
		p += 2;
		/* extension ID=32 */
		Jtest_opus_common.expect_true((packet[p+0] >> 1) == 32, "expected extension id 32", CLASS_NAME, method_name);
		/* For IDs 32 to 127, L=0 signals that the extension data takes up the
		  rest of the padding */
		Jtest_opus_common.expect_true((packet[p+0] & 0x01) == 0, "expected L-bit unset", CLASS_NAME, method_name);
		p += 1;
		Jtest_opus_common.expect_true(0 == Jtest_opus_common.memcmp(packet,p, ext[1].data_base,ext[1].data, ext[1].len), "expected extension content", CLASS_NAME, method_name);
	}

	private static void test_extensions_generate_zero() {
		final String method_name = "test_extensions_generate_zero";
		final byte[] packet = new byte[32];

		/* zero length packet, zero extensions */
		final int result = Jopus_private.opus_packet_extensions_generate( packet, 0, 0, null, 0, 1 );
		Jtest_opus_common.expect_true( result == 0, "expected length 0", CLASS_NAME, method_name );
	}

	private static void test_extensions_generate_no_padding() {
		final String method_name = "test_extensions_generate_no_padding";
		final Jopus_extension_data ext[] = {
				new Jopus_extension_data( 2, 0, "a".getBytes(), 0, 1 ),
				new Jopus_extension_data( 32, 10, "DRED".getBytes(), 0, 4 ),
				new Jopus_extension_data( 33, 1, "NOT DRED".getBytes(), 0, 8 ),
				new Jopus_extension_data( 3, 4, null, 0, 0 )
			};
		final byte[] packet = new byte[32];
 		final int result = Jopus_private.opus_packet_extensions_generate( packet, 0, packet.length, ext, 4, 0 );
 		Jtest_opus_common.expect_true(result == 23, "expected length 23", CLASS_NAME, method_name);
	}

	private static void test_extensions_generate_fail() {
		final String method_name = "test_extensions_generate_fail";
		final Jopus_extension_data ext[] = {
				new Jopus_extension_data( 2, 0, "a".getBytes(), 0, 1 ),
				new Jopus_extension_data( 32, 10, "DRED".getBytes(), 0, 4 ),
				new Jopus_extension_data( 33, 1, "NOT DRED".getBytes(), 0, 8 ),
				new Jopus_extension_data( 3, 4, null, 0, 0 ),
			};
		final byte[] packet = new byte[100];

		/* buffer too small */
		int result = Jopus_private.opus_packet_extensions_generate( packet,0, 4, ext, 4, 1 );
		Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_BUFFER_TOO_SMALL, "expected OPUS_BUFFER_TOO_SMALL", CLASS_NAME, method_name );

		/* invalid id */
		{
			final Jopus_extension_data id_too_big[] = {
					new Jopus_extension_data( 256, 0, "a".getBytes(), 0, 1 ),
				};
			result = Jopus_private.opus_packet_extensions_generate( packet,0, packet.length, id_too_big, 1, 1 );
			Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_BAD_ARG, "expected OPUS_BAD_ARG", CLASS_NAME, method_name );
		}

		/* invalid id */
		{
			final Jopus_extension_data id_too_small[] = {
					new Jopus_extension_data( 1, 0, "a".getBytes(),0, 1 ),
				};
			result = Jopus_private.opus_packet_extensions_generate( packet,0, packet.length, id_too_small, 1, 1 );
			Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_BAD_ARG, "expected OPUS_BAD_ARG", CLASS_NAME, method_name );
		}

		/* frame index too big */
		{
			final Jopus_extension_data frame_too_big[] = {
					new Jopus_extension_data( 33, 48, "a".getBytes(),0, 1 ),
				};
			result = Jopus_private.opus_packet_extensions_generate( packet,0, packet.length, frame_too_big, 1, 1 );
			Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_BAD_ARG, "expected OPUS_BAD_ARG", CLASS_NAME, method_name );
		}

		/* size too big for extension IDs 1 through 31 */
		{
			final Jopus_extension_data size_too_big[] = {
					new Jopus_extension_data( 2, 0, "abcd".getBytes(),0, 4 ),
				};
			result = Jopus_private.opus_packet_extensions_generate( packet,0, packet.length, size_too_big, 1, 1 );
			Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_BAD_ARG, "expected OPUS_BAD_ARG", CLASS_NAME, method_name );
		}

		/* negative size for extension IDs 1 through 31 */
		{
			final Jopus_extension_data neg_size[] = {
					new Jopus_extension_data( 2, 0, null,0, -4),
				};
			result = Jopus_private.opus_packet_extensions_generate( packet,0, packet.length, neg_size, 1, 1 );
			Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_BAD_ARG, "expected OPUS_BAD_ARG", CLASS_NAME, method_name );
		}

		/* negative size for extension IDs 32 through 127 */
		{
			final Jopus_extension_data neg_size_33[] = {
					new Jopus_extension_data( 33, 0, null,0, -4 ),
				};
			result = Jopus_private.opus_packet_extensions_generate( packet,0, packet.length, neg_size_33, 1, 1 );
			Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_BAD_ARG, "expected OPUS_BAD_ARG", CLASS_NAME, method_name );
		}
	}

	private static void test_extensions_parse_success() {
		final String method_name = "test_extensions_parse_success";
		final Jopus_extension_data ext[] = {
				new Jopus_extension_data( 2, 0, "a".getBytes(),0, 1 ),
				new Jopus_extension_data( 32, 10, "DRED".getBytes(),0, 4 ),
				new Jopus_extension_data( 33, 1, "NOT DRED".getBytes(),0, 8 ),
				new Jopus_extension_data( 3, 4, null,0, 0 )
			};
		final Jopus_extension_data[] ext_out = new Jopus_extension_data[10];
		final byte[] packet = new byte[32];

		final int[] nb_ext = new int[] { 10 };
		final int len = Jopus_private.opus_packet_extensions_generate(packet,0, 32, ext, 4, 1);
		Jtest_opus_common.expect_true(len == 32, "expected length 32", CLASS_NAME, method_name );
		int result = Jopus_private.opus_packet_extensions_count(packet,0, len);
		Jtest_opus_common.expect_true(result == 4, "expected opus_packet_extensions_count 4", CLASS_NAME, method_name );
		result = Jopus_private.opus_packet_extensions_parse( packet,0, len, ext_out, 0, nb_ext );
		Jtest_opus_common.expect_true(nb_ext[0] == 4, "expected 4 extensions", CLASS_NAME, method_name );

		Jtest_opus_common.expect_true(ext_out[0].id == 2, "expected id 2", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(ext_out[0].frame == 0, "expected frame 0", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(ext_out[0].len == 1, "expected len 1", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(0 == Jtest_opus_common.memcmp(ext_out[0].data_base,ext_out[0].data, ext[0].data_base,ext[0].data, 1), "expected data", CLASS_NAME, method_name );

		Jtest_opus_common.expect_true(ext_out[1].id == 33, "expected id 33", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(ext_out[1].frame == 1, "expected frame 1", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(ext_out[1].len == 8, "expected len 8", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(0 == Jtest_opus_common.memcmp(ext_out[1].data_base,ext_out[1].data, ext[2].data_base,ext[2].data, 8), "expected data", CLASS_NAME, method_name );

		Jtest_opus_common.expect_true(ext_out[2].id == 3, "expected id 3", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(ext_out[2].frame == 4, "expected frame 4", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(ext_out[2].len == 0, "expected len 0", CLASS_NAME, method_name );

		Jtest_opus_common.expect_true(ext_out[3].id == 32, "expected id 32", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(ext_out[3].frame == 10, "expected frame 10", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(ext_out[3].len == 4, "expected len 4", CLASS_NAME, method_name );
		Jtest_opus_common.expect_true(0 == Jtest_opus_common.memcmp(ext_out[3].data_base,ext_out[3].data, ext[1].data_base,ext[1].data, 4), "expected data", CLASS_NAME, method_name );
	}

	private static void test_extensions_parse_zero() {
		final String method_name = "test_extensions_parse_zero";
		final Jopus_extension_data ext[] = {
				new Jopus_extension_data( 32, 1, "DRED".getBytes(),0, 4 ),
			};
		final byte[] packet = new byte[32];

		final int len = Jopus_private.opus_packet_extensions_generate( packet,0, 32, ext, 1, 1 );
		Jtest_opus_common.expect_true(len == 32, "expected length 32", CLASS_NAME, method_name );

		final int[] nb_ext = new int[] { 0 };
		final int result = Jopus_private.opus_packet_extensions_parse( packet,0, len, null,0, nb_ext );
		Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_BUFFER_TOO_SMALL, "expected OPUS_BUFFER_TOO_SMALL", CLASS_NAME, method_name );
	}

	private static void test_extensions_parse_fail() {
		final String method_name = "test_extensions_parse_fail";
		final Jopus_extension_data ext[] = {
				new Jopus_extension_data( 2, 0, "a".getBytes(),0, 1 ),
				new Jopus_extension_data( 33, 1, "NOT DRED".getBytes(),0, 8 ),
				new Jopus_extension_data( 3, 4, null,0, 0 ),
				new Jopus_extension_data( 32, 10, "DRED".getBytes(),0, 4 )
			};
		final Jopus_extension_data[] ext_out = new Jopus_extension_data[10];
		final byte[] packet = new byte[32];

		/* create invalid length */
		int len = Jopus_private.opus_packet_extensions_generate( packet,0, packet.length, ext, 4, 0 );
		packet[4] = (byte)255;
		final int[] nb_ext = new int[] { 10 };
		int result = Jopus_private.opus_packet_extensions_parse( packet,0, len, ext_out,0, nb_ext );
		Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_INVALID_PACKET, "expected OPUS_INVALID_PACKET", CLASS_NAME, method_name );
		result = Jopus_private.opus_packet_extensions_count( packet,0, len );
		Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_INVALID_PACKET, "expected OPUS_INVALID_PACKET", CLASS_NAME, method_name );

		/* create invalid frame increment */
		nb_ext[0] = 10;
		len = Jopus_private.opus_packet_extensions_generate( packet,0, packet.length, ext, 4, 0);
		packet[14] = (byte)255;
		result = Jopus_private.opus_packet_extensions_parse( packet,0, len, ext_out,0, nb_ext );
		Jtest_opus_common.expect_true(result ==Jopus_defines. OPUS_INVALID_PACKET, "expected OPUS_INVALID_PACKET", CLASS_NAME, method_name );
		/* note, opus_packet_extensions_count does not read the invalid frame increment
		  and tells us that we have 4 extensions */
		result = Jopus_private.opus_packet_extensions_count( packet,0, len );
		Jtest_opus_common.expect_true(result == 4, "expected opus_packet_extensions_count to return 4", CLASS_NAME, method_name );

		/* not enough space */
		nb_ext[0] = 1;
		len = Jopus_private.opus_packet_extensions_generate( packet,0, packet.length, ext, 4, 0 );
		result = Jopus_private.opus_packet_extensions_parse( packet,0, len, ext_out,0, nb_ext );
		Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_BUFFER_TOO_SMALL, "expected OPUS_BUFFER_TOO_SMALL", CLASS_NAME, method_name );
	}

	private static final int NB_RANDOM_EXTENSIONS = 100000000;
	private static final int MAX_EXTENSION_SIZE = 200;
	private static final int MAX_NB_EXTENSIONS = 100;

	private static final void test_random_extensions_parse() {
		final String method_name = "test_random_extensions_parse";
		for( int i = 0; i < NB_RANDOM_EXTENSIONS; i++ ) {
			final Jopus_extension_data[] ext_out = new Jopus_extension_data[MAX_NB_EXTENSIONS];
			final byte[] payload = new byte[MAX_EXTENSION_SIZE];
			final int len = (int)((long)Jtest_opus_common.fast_rand() & 0xFFFFFFFFL) % (MAX_EXTENSION_SIZE + 1);
			for( int j = 0; j < len; j++ ) {
				payload[j] = (byte)(Jtest_opus_common.fast_rand() & 0xFF);
			}
			final int[] nb_ext = new int[] {
					(int)((long)Jtest_opus_common.fast_rand() & 0xFFFFFFFFL) % (MAX_NB_EXTENSIONS + 1)
				};
			final int result = Jopus_private.opus_packet_extensions_parse( payload,0, len, ext_out,0, nb_ext );
			Jtest_opus_common.expect_true(result == Jopus_defines.OPUS_OK || result == Jopus_defines.OPUS_BUFFER_TOO_SMALL || result == Jopus_defines.OPUS_INVALID_PACKET, "expected OPUS_OK, OPUS_BUFFER_TOO_SMALL or OPUS_INVALID_PACKET", CLASS_NAME, method_name );
			/* Even if parsing fails, check that the extensions that got extracted make sense. */
			for( int j = 0; j < nb_ext[0]; j++ ) {
				Jtest_opus_common.expect_true(ext_out[j].frame >= 0 && ext_out[j].frame < 48, "expected frame between 0 and 47", CLASS_NAME, method_name );
				Jtest_opus_common.expect_true(ext_out[j].id >= 2 && ext_out[j].id <= 127, "expected id between 2 and 127", CLASS_NAME, method_name );
				// expect_true(ext_out[j].data >= payload && ext_out[j].data+ext_out[j].len <= payload+len, "expected data to be within packet");
				Jtest_opus_common.expect_true(
						ext_out[j].data >= 0
						&& ext_out[j].data + ext_out[j].len <= len,
						"expected data to be within packet", CLASS_NAME, method_name );
			}
		}
	}

	private static final void test_opus_repacketizer_out_range_impl() {
		final String method_name = "test_opus_repacketizer_out_range_impl";
		int first_count = 0, second_count = 0;
		final Jopus_extension_data[] ext = {
				new Jopus_extension_data( 33, 0, "abcdefg".getBytes(),0, 7 ),
				new Jopus_extension_data( 100, 0, "uvwxyz".getBytes(),0, 6 ),
			};

		final JOpusRepacketizer rp = new JOpusRepacketizer();
		rp.opus_repacketizer_init();

		final byte[] packet = new byte[1024];
		Arrays.fill( packet, (byte) 0 );
		/* Hybrid Packet with 20 msec frames, Code 3 */
		packet[0] = (15 << 3) | 3;
		/* Code 3, padding bit set, 1 frame */
		packet[1] = 1 << 6 | 1;
		packet[2] = 0;
		packet[3] = 0;

		/* generate 2 extensions, id 33 and 100 */
		final int len = Jopus_private.opus_packet_extensions_generate( packet,4, packet.length-4, ext, 2, 0 );
		/* update the padding length */
		packet[2] = (byte) len;

		/* concatenate 3 frames */
		int res = rp.opus_repacketizer_cat( packet,0, 4+len );
		/* for the middle frame, no padding, no extensions */
		packet[1] = 1;
		res = rp.opus_repacketizer_cat( packet,0, 4 );
		/* switch back to extensions for the last frame extensions */
		packet[1] = 1 << 6 | 1;
		res = rp.opus_repacketizer_cat( packet,0, 4+len );

		Jtest_opus_common.expect_true( rp.nb_frames == 3, "Expected 3 frames", CLASS_NAME, method_name );
		final byte[] packet_out = new byte[1024];
		res = rp.opus_repacketizer_out_range_impl(
				0, 3, /* begin, end */
				packet_out, 0, /* unsigned char *data */
				packet_out.length, /* opus_int32 maxlen */
				false, /*int self_delimited */
				false, /* int pad */
				null, /* const opus_extension_data *extensions */
				0 /* int nb_extensions */);
		Jtest_opus_common.expect_true(res > 0, "expected valid packet length", CLASS_NAME, method_name );

		/* now verify that we have the expected extensions */
		final short[] size = new short[48];
		final Jopus_packet_data_aux aux = new Jopus_packet_data_aux();// java
		res = Jopus.opus_packet_parse_impl( packet_out,0, res, false,/* null,*/ null,0, size,0,
				aux );// null, null, &padding, &padding_len );
		final byte[] padding_base = packet_out;// java
		final int padding = aux.mPadding;
		final int padding_len = aux.mPaddingLen;

		final int[] nb_ext = new int[] { 10 };
		final Jopus_extension_data[] ext_out = new Jopus_extension_data[10];
		res = Jopus_private.opus_packet_extensions_parse( padding_base,padding, padding_len, ext_out,0, nb_ext );
		Jtest_opus_common.expect_true( nb_ext[0] == 4, "Expected 4 extensions", CLASS_NAME, method_name );
		for( int i = 0 ; i < nb_ext[0]; i++ ) {
			if( ext_out[i].id == 33 ) {
				Jtest_opus_common.opus_test_assert(ext_out[i].len == ext[0].len, CLASS_NAME, method_name );
				Jtest_opus_common.opus_test_assert(0 == Jtest_opus_common.memcmp(ext_out[i].data_base,ext_out[i].data, ext[0].data_base,ext[0].data, ext[0].len), CLASS_NAME, method_name );
				first_count++;
			}
			else if( ext_out[i].id == 100 ) {
				Jtest_opus_common.opus_test_assert( ext_out[i].len == ext[1].len, CLASS_NAME, method_name );
				Jtest_opus_common.opus_test_assert( 0 == Jtest_opus_common.memcmp(ext_out[i].data_base,ext_out[i].data, ext[1].data_base,ext[1].data, ext[1].len), CLASS_NAME, method_name );
				second_count++;
			}
			if( i < 2 ) {
				Jtest_opus_common.opus_test_assert( ext_out[i].frame == 0, CLASS_NAME, method_name );
			} else {
				Jtest_opus_common.opus_test_assert( ext_out[i].frame == 2, CLASS_NAME, method_name );
			}
		}
		Jtest_opus_common.opus_test_assert( first_count == 2, CLASS_NAME, method_name );
		Jtest_opus_common.opus_test_assert( second_count == 2, CLASS_NAME, method_name );
	}

	@SuppressWarnings("boxing")
	public static final void main(final String argv[])
	{
		boolean env_used = false;
		final String env_seed = System.getenv("SEED");
		if( argv.length > 1 - 1 ) {// java -1, no path
			try {
				Jtest_opus_common.iseed = Integer.parseInt( argv[1 - 1] );// java -1, no path
			} catch( final NumberFormatException ne ) {
				Jtest_opus_common.iseed = 0;
			}
		} else if( null != env_seed ) {
			try {
				Jtest_opus_common.iseed = Integer.parseInt( env_seed );
			} catch( final NumberFormatException ne ) {
				Jtest_opus_common.iseed = 0;
			}
			env_used = true;
		} else {
			Jtest_opus_common.iseed = (int)System.currentTimeMillis();
		}
		Jtest_opus_common.Rw = Jtest_opus_common.Rz = Jtest_opus_common.iseed;

		System.err.printf("Testing extensions. Random seed: %d (%04X)%n",
				Jtest_opus_common.iseed, ((long)Jtest_opus_common.fast_rand() & 0xffffffffL) % 65535 );
		if( env_used ) {
			System.err.printf("  Random seed set from the environment (SEED=%s).\n", env_seed );
		}

		test_extensions_generate_success();
		test_extensions_generate_zero();
		test_extensions_generate_no_padding();
		test_extensions_generate_fail();
		test_extensions_parse_success();
		test_extensions_parse_zero();
		test_extensions_parse_fail();
		test_random_extensions_parse();
		test_opus_repacketizer_out_range_impl();
		System.err.printf("Tests completed successfully.%n");
		System.exit( 0 );
		return;
	}
}
